# Copyright (c) 2003-2004 Chalmers University of Technology
# Authors: Jean-Marc Orliaguet <jmo@ita.chalmers.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

__author__ = "Jean-Marc Orliaguet <jmo@ita.chalmers.se>"

"""
  Portal Theme
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.PortalFolder import PortalFolder

from ThemeFolder import ThemeFolder
from cpsskins_utils import rebuild_properties, callAction, \
                           getFreeId, verifyThemePerms, canonizeId

import string

def addPortalTheme(dispatcher, id, REQUEST=None):
    """Add a Portal Theme."""
    ob = PortalTheme(id)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'Portal Theme',
     'title': 'Portal Theme',
     'description': ('_portaltheme_description_'),
     'meta_type': 'Portal Theme',
     'icon': 'portaltheme.gif',
     'product': 'CPSSkins',
     'factory': 'addPortalTheme',
     'immediate_view': 'cpsskins_edit_form',
     'filter_content_types': 0,
     'global_allow': 0,
     'actions': (
                 {'id': 'view',
                  'name': 'View',
                  'action': 'cpsskins_default_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_Edit_',
                  'action': 'cpsskins_edit_form',
                  'visible': 0,
                  'permissions': ('Manage Themes',)
                 },
                 {'id': 'editstyles',
                  'name': '_Edit styles_',
                  'action': 'cpsskins_edit_styles',
                  'visible': 0,
                  'permissions': ('Manage Themes',)
                 },
                )
     },
    )


class PortalTheme(ThemeFolder):
    """
    Class for Portal Themes.
    """

    meta_type = "Portal Theme"
    portal_type = "Portal Theme"

    manage_options = ( 
                      Folder.manage_options[0:1] +
                      Folder.manage_options[2:3] +
                      (
                       {'label': 'Rebuild',
                        'action': 'manage_themeRebuild'},
                       ) 
                     )

    security = ClassSecurityInfo()

    security.declareProtected('Manage Themes', 'manage_themeRebuild')
    manage_themeRebuild = DTMLFile('zmi/manage_themeRebuild', globals())


    _properties = (
        {'id': 'title', 
         'type':'string', 
         'mode':'w', 
         'label':'Title', 
         'category': 'general'
        },
        {'id': 'margin', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'Margin', 
         'category': 'general'
        },
        {'id': 'default', 
         'type': 'boolean', 
         'mode': 'w', 
         'label': 'Default', 
         'category': 'none',
         'default': 0,
        },
        {'id': 'align', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Align', 
         'select_variable': 'cpsskins_alignList', 
         'category': 'general'
        },
        {'id': 'color', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Color', 
         'select_variable': 'cpsskins_listAreaColors', 
         'meta_type': 'Area Color',
         'category': 'general'
        },
        {'id': 'shortcut_icon', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Shortcut icon', 
         'select_variable': 'cpsskins_listIcons',  
         'category': 'general', 
         'image' : 'icons'
        },
        {'id': 'theme_renderer', 
         'type': 'selection', 
         'mode': 'w', 
         'label': 'Theme renderer', 
         'select_variable' : 'cpsskins_listThemeRenderers',  
         'category': 'general' 
        },
        {'id': 'author', 
         'type': 'text', 
         'mode': 'w', 
         'label': 'Author', 
         'category' : 'general'
        },
        {'id': 'copyright', 
         'type': 'text', 
         'mode': 'w', 
         'label': 'Copyright', 
         'category': 'general'
        },
        {'id': 'license', 
         'type': 'string', 
         'mode': 'w', 
         'label': 'License', 
         'category': 'general'
        },
    )

    def __init__(self, id, 
                       title = 'Portal Theme',
                       color = 'Transparent',
                       align = 'center',
                       margin = '0.5em',
                       default = 0,
                       shortcut_icon = '',
                       theme_renderer = 'default',
                       author = '',
                       copyright = '',
                       license = '',
                       **kw):
        self.id = id
        self.title = title
        self.color = color
        self.align = align
        self.margin = margin
        self.default = default
        self.shortcut_icon = shortcut_icon
        self.theme_renderer = theme_renderer
        self.author = author
        self.copyright = copyright
        self.license = license


    security.declareProtected(View,'isPortalTheme')
    def isPortalTheme(self):
        """ is a portal theme """
           
        return 1


    security.declareProtected(View,'isPortalContent')
    def isPortalContent(self):
        """ is portal content """
                  
        return 0  


    security.declareProtected('Manage Themes', 'manage_rebuild')
    def manage_rebuild(self, REQUEST=None):
        """
        Rebuild this theme
        """

        self.rebuild()
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')


    security.declareProtected('Manage Themes', 'createThemeSkeleton')
    def createThemeSkeleton(self):
        """
        Creates a theme skeleton
        """

        themefolders = [ 'icons', 'styles', 'backgrounds', 'palettes' ]

        for themefolder in themefolders:
            self.invokeFactory('Theme Folder', id=themefolder)


    security.declareProtected('Manage Themes', 'rebuild')
    def rebuild(self):
        """
        Rebuild the theme
        """

        canonizeId(self)

        themefolders = [ 'icons', 'styles', 'backgrounds', 'palettes' ]

        for themefolder in themefolders:
            backupid = themefolder + '-bak'
            objs = self.objectIds()
            if backupid in objs:
                self.manage_delObjects([backupid])

            if themefolder not in objs:
                self.invokeFactory('Theme Folder', id=themefolder)
                continue
        
            else:
                folder = getattr(self.aq_explicit, themefolder)
                meth = getattr(folder.aq_explicit, 'isThemeFolder', None)
                if callable(meth) and meth():
                    continue

                self.manage_renameObjects([themefolder], [backupid])
                backup_folder = getattr(self.aq_explicit, backupid)
                objects = backup_folder.objectIds()
                cookie = backup_folder.manage_cutObjects(objects)
                self.invokeFactory('Theme Folder', themefolder)
                new_folder = getattr(self.aq_explicit, themefolder)
                new_folder.manage_pasteObjects(cookie)
                self.manage_delObjects([backupid])

        rebuild_properties(self) 

        for obj in self.getPageBlocks():
            try:
                obj.rebuild()
            except TypeError:
                obj.aq_parent.manage_delObjects([obj.getId()])

        for obj in self.findStyles():
            try:
                obj.rebuild()
            except TypeError:
                obj.aq_parent.manage_delObjects([obj.getId()])

        for themefolder in themefolders:
            obj = getattr(self.aq_explicit, themefolder, None)
            if obj is not None:
                 verifyThemePerms(obj)
        verifyThemePerms(self)


    security.declarePublic('getPageBlocks')
    def getPageBlocks(self):
        """ returns a list of page blocks sorted by ypos"""

        return self.objectValues('Page Block')


    security.declarePublic('getStylesFolder')
    def getStylesFolder(self):
        """
        Returns the styles folder object 
        """

        return getattr(self, 'styles', None)


    security.declarePublic('findStyles')
    def findStyles(self, title=None, meta_type=None):
        """
        Returns a list of style(s) that has/have a given title or meta type
        """

        styles_dir = self.getStylesFolder()
        list = []
        for style in styles_dir.objectValues():
            if title is not None:
               if style.getTitle() != title:
                  continue

            if meta_type is not None:
               if getattr(style, 'meta_type', None) != meta_type:
                  continue

            meth = getattr(style.aq_explicit, 'isPortalStyle', None)
            if callable(meth) and meth():
               list.append(style)
 
        return list 

        
    
    security.declarePublic('findIdenticalStyles')
    def findIdenticalStyles(self):
        """
        Returns a dictionary of identical styles by type
        """

        dict = {}
        tmtool = self.portal_themes
        for style_type in tmtool.listStyleTypes():
            meta_type = style_type.getId()
            identical_styles = self.findIdenticalStylesByType(meta_type=meta_type)
            if identical_styles:
                dict[meta_type] = identical_styles

        return dict


    security.declarePublic('findIdenticalStylesByType')
    def findIdenticalStylesByType(self, meta_type=None):
        """
        Returns a list of a list of styles that have identical properties
        """

        if meta_type is None:
            return 
 
        styles = self.findStyles(meta_type=meta_type)

        groups = []
        props = []

        for style in styles:      
            properties =  style.propertyValues()[1:]
            for i in range(len(groups)):
                group = groups[i]
                if properties == props[i]: 
                     groups[i].append(style)
                     break
    
            groups.append([style])
            props.append(properties)

        final_groups = []
        for group in groups:
            if len(group) > 1:
                final_groups.append(group)

        return final_groups


    security.declareProtected('Manage Themes', 'addPageBlock')
    def addPageBlock(self, **kw):
        """
        Add a Page Block. Returns the Page Block's id
        """

        id = getFreeId(self)
        self.invokeFactory('Page Block', title='PageBlock', id=id)
        pageblock = getattr(self, id, None) 

        if pageblock is not None:
           pageblock_ypos = kw.get('pageblock_ypos', 0)

           self.move_object_to_position(pageblock.getId(), newpos=int(pageblock_ypos)+1)
           return pageblock
        else:
           return


    security.declareProtected('Manage Themes', 'addPortalPalette')
    def addPortalPalette(self, **kw):
        """
        Add a Portal Palette. Returns the Portal Palette's id
        """
      
        type_name = kw.get('type_name', None)
        if type_name is None:
            return
        del kw['type_name']

        palettes_dir = getattr(self, 'palettes', None)
        if palettes_dir is None:
            return None

        titles = [getattr(obj, 'title', None) for obj in palettes_dir.objectValues()]

        type = string.replace(type_name, ' ', '') 
        i = 0
        while 1:
           i = i + 1
           title = type + str(i)
           if titles is None: 
               break
           if title not in titles:
               break 

        id = getFreeId(self)
        palettes_dir.invokeFactory(type_name, id, title=title, **kw)

        palette = getattr(palettes_dir, id, None) 
        return palette


    security.declareProtected('Manage Themes', 'addPortalStyle')
    def addPortalStyle(self, **kw):
        """
        Add a Portal Style. Returns the Portal Style's id
        """
      
        type_name = kw.get('type_name', None)
        if type_name is None:
            return
        del kw['type_name']

        styles_dir = getattr(self, 'styles', None)
        if styles_dir is None:
            return None

        titles = [getattr(obj, 'title', None) for obj in styles_dir.objectValues()]

        type = string.replace(type_name, ' ', '') 
        i = 0
        while 1:
           i = i + 1
           title = type + str(i)
           if titles is None: 
               break
           if title not in titles:
               break 

        id = getFreeId(self)
        styles_dir.invokeFactory(type_name, id, title=title, **kw)
                                                
        styles = getattr(styles_dir, id, None) 
        return styles


    security.declareProtected('Manage Themes', 'addPortalImage')
    def addPortalImage(self, **kw):
        """
        Add a Portal Image. Returns the Portal Image's id
        """

        file = kw.get('file', None)
        if file is None:
            return

        imagecat = kw.get('imagecat', None)
        if imagecat is None:
            return
        
        title = file.filename
        id = title

        images_dir = getattr(self, imagecat, None)
        if images_dir is None:
            return None

        ids = images_dir.objectIds()

        prefix = title.split('.')[0]
        ext = title.split('.')[1]

        i = 0
        while 1:
           i = i + 1
           if ids is None: 
               break
           if id not in ids:
               break 
           id = prefix + str(i) + '.' + ext

        title = id
        images_dir.manage_addProduct['CMFDefault'].manage_addContent(id=id, type='Portal Image')

        img = getattr(images_dir, id, None)
        img.manage_upload(file)
        img.manage_changeProperties(title=title)
 
        return img


    security.declareProtected('Manage Themes', 'findOrphanedStyles')
    def findOrphanedStyles(self, **kw):
        """
        Returns the list of styles that are not used.
        """

        stylesfolder = self.getStylesFolder()
        list = []
        for style in stylesfolder.objectValues():
             if hasattr(style.aq_explicit, 'isOrphan'):
                 if style.isOrphan():
                     list.append(style)
        return list


    security.declareProtected('Manage Themes', 'getInvisibleTemplets')
    def getInvisibleTemplets(self, **kw):
        """
        Returns the list of invisible templets.
        """

        pageblocks = self.getPageBlocks()
        invisible_templets = []
        for pageblock in pageblocks:
             maxcols = getattr(pageblock, 'maxcols', None)
             pageblock_closed = pageblock.closed
             if maxcols is not None:
                 maxcols = int(maxcols)
             for obj in pageblock.objectValues():
                 meth = getattr(obj.aq_explicit, 'isPortalTemplet', None)
                 if callable(meth) and meth():
                      if obj.closed:
                          invisible_templets.append(obj)
                          continue
                      if pageblock_closed:
                          invisible_templets.append(obj)
                          continue
                      xpos = getattr(obj, 'xpos', None)
                      if xpos is not None and xpos >= maxcols:
                          invisible_templets.append(obj)

        return invisible_templets

        

    security.declareProtected('Manage Themes', 'edit_form')
    def edit_form(self, **kw):
        """
        Call the edit action.
        """
        return callAction(self, 'edit', **kw)


    security.declareProtected('Manage Themes', 'edit')
    def edit(self, **kw):
        """
        Default edit method, changes the properties.
        """
        self.manage_changeProperties(**kw)


InitializeClass(PortalTheme)


